# 获取脚本所在的目录
CUR_DIR=$(dirname $0)
# 进入到该目录运行,这样可以规避绝对路径的问题,你的脚本迁移到任何目录都可以正常运行
cd ${CUR_DIR}

# 该函数running正常退出,not running则退出非0值
check_server_is_running() {
    [[ ! -f "pid" ]] && touch pid
    
    # 获取
    PID=$(cat pid)
    
    # 首先判断pid是否存在
    if [ "${PID}" = "" ]
    then
        echo "server is not running."
        return 1
    fi
    
    # 再判断指定的PID是否在当前目录运行,防止因历史异常原因遗留了失效PID,正好该PID又被其他进程占用,造成误杀。
    if [[ -d /proc/${PID}/cwd ]] && ls -ahl /proc/${PID}/cwd | grep -q "${CUR_DIR}"
    then
        echo "server is running."
        return 0
    else
        # 清空失效的进程号并异常退出
        echo "" > pid
        return 1
    fi
}

start() {
    # 启动前判断日志目录是否存在,不存在就创建
    [[ ! -d "../logs" ]] && mkdir -p ../logs
    
    # 启动服务
    sh test.sh > ../logs/start.log 2>&1 &
    
    # 保存PID,以供stop使用
    PID=$!
    echo "server start succ, PID=${PID}!"
    echo "${PID}" > pid
}



stop() {
    # get pid
    PID=$(cat pid)
    kill -9 ${PID}
}


case C"$1" in
    Cstart)
        check_server_is_running || start
        echo "start done."
        ;;
    Cstop)
        check_server_is_running && stop
        echo "stop done."
        ;;
    Crestart)
        check_server_is_running && stop || start
        echo "restart done."
        ;;
    C*)
        echo "Usage: $0 {start|stop|restart}"
        ;;
esac
